/*
** isc_handler.c                 Routines to handle IscSession handlers
**
** Copyright (c) 1992 Peter Eriksson and Per Cederqvist of the
**                    Lysator Academic Computer Association.
**
** history:
** 920211 pen      initial coding
*/

#include <stdio.h>
#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>

#include "isc.h"
#include "intern.h"



IscHandler *
isc_copyhandler(IscHandler *hcb)
{
  IscHandler *nhcb;


  ISC_XNEW(nhcb);
  *nhcb = *hcb;

  return nhcb;
}



IscHandler *
isc_newhandler(void)
{
  IscHandler *hcb;


  ISC_XNEW(hcb);
  
  hcb->read = NULL;
  hcb->write = NULL;
  hcb->close = NULL;
  hcb->poll = NULL;
  hcb->accept = NULL;
  hcb->destroy = NULL;
  hcb->parse = NULL;
  
  return hcb;
}



void
isc_freehandler(IscHandler *hcb)
{
  isc_free(hcb);
}



void
isc_pushhandler(IscSession *scb, IscHandler *hcb)
{
  IscHandlerList *ihl;

  
  ISC_XNEW(ihl);
  ihl->hcb = hcb;
  ihl->old = scb->fun;
  ihl->next = scb->handlers;
  
  scb->handlers = ihl;

  if (hcb->read)
    scb->fun.read = ihl;
  if (hcb->write)
    scb->fun.write = ihl;
  if (hcb->close)
    scb->fun.close = ihl;
  if (hcb->poll)
    scb->fun.poll = ihl;
  if (hcb->accept)
    scb->fun.accept = ihl;
  if (hcb->destroy)
    scb->fun.destroy = ihl;
  if (hcb->parse)
    scb->fun.parse = ihl;
}


IscHandler *
isc_pophandler(IscSession *scb)
{
  IscHandlerList *ip;
  IscHandler *hcb;

  
  if (scb->handlers == NULL)
    return NULL;

  ip = scb->handlers;
  scb->handlers = ip->next;
  scb->fun = ip->old;
  hcb = ip->hcb;
    
  isc_free(ip);
  
  return hcb;
}



