/*
** isc_master.c                               IscMaster control functions
**
** Copyright (c) 1992 Peter Eriksson and Per Cederqvist of the
**                    Lysator Academic Computer Association.
**
** history:
** 910305 pen      moved to separate file
*/

#include <errno.h>
#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif
#ifdef HAVE_STDDEF_H
#  include <stddef.h>
#endif
#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#ifndef NULL
#  include <stdio.h>
#endif

#include "isc.h"
#include "intern.h"



/*
** This routine will initialize the ISC subsystem
*/
IscMaster *
isc_initialize(IscConfig  * cfg)
{
  IscMaster         * mcb;
  IscMasterConfig   * mcfg;
  IscSessionConfig  * scfg;

  
  /* Handle user specified defaults */
  if (cfg)
    switch (cfg->version)
    {
      case 0:
        break;
	
      case 1005:
	mcfg = &cfg->master;
	
	switch (mcfg->version)
	{
	  case 0:
	    break;
	    
	  case 1001:
	    if (mcfg->memfn.alloc &&
		mcfg->memfn.realloc &&
		mcfg->memfn.free)
	      isc_setallocfn(mcfg->memfn.alloc,
			     mcfg->memfn.realloc,
			     mcfg->memfn.free);
	    if (mcfg->abortfn)
	      isc_setabortfn(mcfg->abortfn);
	    break;

	  default:
	    errno = EINVAL;
	    return NULL;
	}
	break;

      default:
	errno = EINVAL;
	return NULL;
    }
  
  ISC_XNEW(mcb);
  
  mcb->sessions = NULL;
  
  
  /* Setup default values */
  mcb->scfg.max.msgsize     = ISC_DEFAULT_MAX_MSG_SIZE;
  mcb->scfg.max.queuedsize  = ISC_DEFAULT_MAX_QUEUED_SIZE;
  mcb->scfg.max.dequeuelen  = ISC_DEFAULT_MAX_DEQUEUE_LEN;
  mcb->scfg.max.openretries = ISC_DEFAULT_MAX_OPEN_RETRIES;
  mcb->scfg.max.backlog     = ISC_DEFAULT_MAX_BACKLOG;
  
  /* Handle user specified defaults */
  if (cfg)
    switch (cfg->version)
    {
      case 1005:
        scfg = &cfg->session;
	
	switch (scfg->version)
	{
	  case 0:
	    break;
	    
	  case 1001:
	    if (scfg->max.msgsize >= 0)
	      mcb->scfg.max.msgsize = scfg->max.msgsize;
	    if (scfg->max.queuedsize >= 0)
	      mcb->scfg.max.queuedsize = scfg->max.queuedsize;
	    if (scfg->max.dequeuelen >= 0)
	      mcb->scfg.max.dequeuelen = scfg->max.dequeuelen;
	    if (scfg->max.openretries >= 0)
	      mcb->scfg.max.openretries = scfg->max.openretries;
	    if (scfg->max.backlog >= 0)
	      mcb->scfg.max.backlog = scfg->max.backlog;
	    break;

	  default:
	    isc_free(mcb);
	    errno = EINVAL;
	    return NULL;
	}

	break;
	
      default:
	isc_free(mcb);
	errno = EINVAL;
	return NULL;
    }
  
  return mcb;
}



/*
** Close and destroy all sessions associated with a MCB, then
** destroy the MCB too.
*/
void
isc_shutdown(IscMaster  * mcb)
{
  while (mcb->sessions)
    isc_destroy(mcb, mcb->sessions->scb);

  isc_free(mcb);
}
